package io.xjar;

import io.xjar.filter.XMixEntryFilter;
import java.util.regex.Pattern;
import java.security.NoSuchAlgorithmException;
import io.xjar.filter.XAllEntryFilter;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import io.xjar.filter.XAnyEntryFilter;
import io.xjar.key.XKey;
import java.io.File;

public class XEncryption
{
    private File jar;
    private XKey key;
    private XAnyEntryFilter<JarArchiveEntry> includes;
    private XAllEntryFilter<JarArchiveEntry> excludes;
    
    public XEncryption() {
        this.includes = XKit.any();
        this.excludes = XKit.all();
    }
    
    public XEncryption from(final String jar) {
        return this.from(new File(jar));
    }
    
    public XEncryption from(final File jar) {
        this.jar = jar;
        return this;
    }
    
    public XEncryption use(final String password) {
        try {
            this.key = XKit.key(password);
            return this;
        }
        catch (final NoSuchAlgorithmException e) {
            throw new IllegalStateException(e);
        }
    }
    
    public XEncryption use(final String algorithm, final int keysize, final int ivsize, final String password) {
        try {
            this.key = XKit.key(algorithm, keysize, ivsize, password);
            return this;
        }
        catch (final NoSuchAlgorithmException e) {
            throw new IllegalStateException(e);
        }
    }
    
    public XEncryption include(final String ant) {
        this.includes.mix((XEntryFilter)XFilters.ant(ant));
        return this;
    }
    
    public XEncryption include(final Pattern regex) {
        this.includes.mix((XEntryFilter)XFilters.regex(regex.pattern()));
        return this;
    }
    
    public XEncryption exclude(final String ant) {
        this.excludes.mix((XEntryFilter)XFilters.not((XEntryFilter<JarArchiveEntry>)XFilters.ant(ant)));
        return this;
    }
    
    public XEncryption exclude(final Pattern regex) {
        this.excludes.mix((XEntryFilter)XFilters.not((XEntryFilter<JarArchiveEntry>)XFilters.regex(regex.pattern())));
        return this;
    }
    
    public void to(final String xJar) throws Exception {
        this.to(new File(xJar));
    }
    
    public void to(final File xJar) throws Exception {
        if (this.jar == null) {
            throw new IllegalArgumentException("jar to encrypt is null. [please call from(String jar) or from(File jar) before]");
        }
        if (this.key == null) {
            throw new IllegalArgumentException("key to encrypt is null. [please call use(String password) or use(String algorithm, int keysize, int ivsize, String password) before]");
        }
        XMixEntryFilter<JarArchiveEntry> filter;
        if (this.includes.size() == 0 && this.excludes.size() == 0) {
            filter = null;
        }
        else {
            filter = (XMixEntryFilter<JarArchiveEntry>)XKit.all();
            if (this.includes.size() > 0) {
                filter.mix((XEntryFilter)this.includes);
            }
            if (this.excludes.size() > 0) {
                filter.mix((XEntryFilter)this.excludes);
            }
        }
        XCryptos.encrypt(this.jar, xJar, this.key, (XEntryFilter<JarArchiveEntry>)filter);
    }
}
